<?php
/**
 * ownCloud
 *
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @author Thomas Müller <thomas.mueller@tmit.eu>
 * @copyright (C) 2014-2017 ownCloud, GmbH.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\ObjectStore;
use Aws\Handler\GuzzleV5\GuzzleHandler;
use Aws\S3\Exception\S3Exception;
use Aws\S3\ObjectUploader;
use Aws\S3\S3Client;
use GuzzleHttp\Event\BeforeEvent;
use GuzzleHttp\Ring\Client\StreamHandler;
use OC\ServiceUnavailableException;
use OCP\Files\ObjectStore\IObjectStore;

require_once __DIR__ . '/../vendor/autoload.php';

class S3 implements IObjectStore {

	/**
	 * @var S3Client
	 */
	private $connection;

	/**
	 * @var array
	 */
	private $params;

	/**
	 * S3Storage constructor.
	 *
	 * @param $params
	 * @throws \Exception
	 */
	public function __construct($params) {

		$this->mapLegacyConfig($params);

		if (!isset($params['options']) || !isset($params['bucket']) ) {
			throw new \Exception('Connection options and bucket must be configured.');
		}

		$this->params = $params;
	}

	public function mapLegacyConfig (array &$params) {
		if (!isset($params['options'])) {
			$logger = \OC::$server->getLogger();
			$logger->warning('Please update your s3 config and provide an options element. Trying to map legacy config (key, secret and region).');
			if (!isset($params['key']) || !isset($params['secret']) ) {
				throw new \Exception('Access Key and Secret must be configured.');
			}
			$params['options'] = [
				'credentials' => [
					'key' => $params['key'],
					'secret' => $params['secret'],
				]
			];
			if (isset($params['region'])) {
				$params['options']['region'] = $params['region'];
			}
		}
		// fix migration of path style option
		if (isset($params['options']['command.params']['PathStyle'])) {
			$params['options']['use_path_style_endpoint'] = $params['options']['command.params']['PathStyle'];
			unset($params['options']['command.params']);
		}
	}

	protected function init() {
		if ($this->connection) {
			return;
		}
		$config = $this->params['options'];
		$client = new \GuzzleHttp\Client(['handler' => new StreamHandler()]);
		$emitter = $client->getEmitter();
		$emitter->on('before', function (BeforeEvent $event) {
			$request = $event->getRequest();
			if ($request->getMethod() !== 'PUT') {
				return;
			}
			$body = $request->getBody();
			if ($body !== null && $body->getSize() !== 0) {
				return;
			}
			if ($request->hasHeader('Content-Length')) {
				return;
			}
			// force content length header on empty body
			$request->setHeader('Content-Length', 0);
		});
		$h = new GuzzleHandler($client);
		$config['http_handler'] = $h;
		$this->connection = S3Client::factory($config);
		try {
			$this->connection->listBuckets();
		} catch(S3Exception $exception) {
			\OC::$server->getLogger()->logException($exception);
			throw new ServiceUnavailableException("No S3 ObjectStore available");
		}

		$this->connection->registerStreamWrapper();

		if ($this->params['autocreate'] && !$this->connection->doesBucketExist($this->params['bucket'])) {
			try {
				$this->connection->createBucket([
					'Bucket' => $this->params['bucket']
				]);
				$this->connection->waitUntil('BucketExists', [
					'Bucket' => $this->params['bucket'],
					'waiter.interval' => 1,
					'waiter.max_attempts' => 15
				]);
			} catch (S3Exception $e) {
				\OC::$server->getLogger()->logException($e, ['app' => 'objectstore']);
				throw new \Exception('Creation of bucket failed. '.$e->getMessage());
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public function getStorageId() {
		return $this->params['bucket'];
	}

	/**
	 * {@inheritDoc}
	 */
	public function writeObject($urn, $stream) {
		$this->init();

		$opt = [];
		if (isset($this->params['serversideencryption'])) {
			$opt['ServerSideEncryption'] = $this->params['serversideencryption'];
		}

		$uploader = new ObjectUploader($this->connection, $this->params['bucket'], $urn, $stream, 'private', $opt);
		$uploader->upload();
		if (is_resource($stream)) {
			fclose($stream);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public function deleteObject($urn) {
		$this->init();
		$this->connection->deleteObject([
			'Bucket' => $this->params['bucket'],
			'Key' => $urn,
		]);
	}

	/**
	 * {@inheritDoc}
	 */
	public function readObject($urn) {
		$this->init();
		return fopen($this->getUrl($urn), 'r');
	}

	public function getUrl($urn) {
		return 's3://'.$this->params['bucket'].'/'.$urn;
	}
}
